import { reactive, ref } from 'vue'

export const functions_store = reactive({
  functions: ref(
    [] as Array<{
      name: string
      tags: {
        id: string
        created_at: string
        updated_at: string
        deleted_at: string
        name: string
      }[]
      requirements: {
        id: string
        created_at: string
        updated_at: string
        deleted_at: string
        name: string
      }[]
    }>,
  ),
})

export function get_functions() {
  fetch('http://192.168.50.91:31337/all/functions', {
    credentials: 'include',
  })
    .then((res) => {
      res
        .json()
        .then((functionsjson) => {
          console.log(functionsjson)
          functions_store.functions = functionsjson.functions
        })
        .catch(() => {})
    })
    .catch(() => {})
}

export function update_function(
  function_name: string,
  function_tags: string[],
  function_requirements: string[],
) {
  const post_url = new URL('http://192.168.50.91:31337/function')
  const search_params = new URLSearchParams()
  search_params.append('id', function_name)
  function_tags.forEach((tag) => {
    search_params.append('tags', tag)
  })
  if (function_requirements.length > 0 && function_requirements[0] != 'undefined') {
    function_requirements.forEach((requirement) => {
      search_params.append('requirements', requirement)
    })
  }
  post_url.search = search_params.toString()
  fetch(post_url, {
    credentials: 'include',
    method: 'PUT',
    body: JSON.stringify({
      name: function_name,
      tags: function_tags,
      requirements: function_requirements,
    }),
  })
    .then((res) => {
      if (res.status === 400) {
        console.log('Function submitted is invalid.')
      } else if (res.status === 401) {
        console.log("You aren't logged in.")
      } else {
        get_functions()
      }
    })
    .catch(() => {})
}

export function delete_function(function_name: string) {
  const delete_url = new URL('http://192.168.50.91:31337/function')
  const search_params = new URLSearchParams()
  search_params.append('id', function_name)
  delete_url.search = search_params.toString()
  fetch(delete_url, {
    credentials: 'include',
    method: 'DELETE',
    body: JSON.stringify({
      name: function_name,
    }),
  })
    .then((res) => {
      if (res.status === 400) {
        console.log('Function submitted is invalid.')
      } else if (res.status === 401) {
        console.log("You aren't logged in.")
      } else {
        get_functions()
      }
    })
    .catch(() => {})
}

export function add_function(
  function_name: string,
  function_tags: string[],
  function_requirements: string[],
) {
  const post_url = new URL('http://192.168.50.91:31337/function')
  const search_params = new URLSearchParams()
  search_params.append('id', function_name)
  function_tags.forEach((tag) => {
    search_params.append('tags', tag)
  })
  if (function_requirements.length > 0 && function_requirements[0] != 'undefined') {
    function_requirements.forEach((requirement) => {
      search_params.append('requirements', requirement)
    })
  }
  post_url.search = search_params.toString()
  fetch(post_url, {
    credentials: 'include',
    method: 'POST',
    body: JSON.stringify({
      name: function_name,
      tags: function_tags,
      requirements: function_requirements,
    }),
  })
    .then((res) => {
      if (res.status === 400) {
        console.log('Function submitted is invalid.')
      } else if (res.status === 401) {
        console.log("You aren't logged in.")
      } else {
        get_functions()
      }
    })
    .catch(() => {})
}
